@@ -56,8 +56,8 @@ void Log_info_client(client_t *client, const char *logstring, ...);
 void Log_fatal(const char *logstring, ...) __attribute__((__noreturn__));
 
 void Log_init(bool_t terminal);
-void Log_reset();
-void Log_free();
+void Log_reset(void);
+void Log_free(void);
 
 #endif
 
