@@ -37,10 +37,16 @@
 #include <stdlib.h>
 #include <fcntl.h>
 
-#include <mbedtls/config.h>
 #include <mbedtls/version.h>
-#include <mbedtls/havege.h>
+#if defined(MBEDTLS_USE_PSA_CRYPTO)
+#include <mbedtls/psa_util.h>
+#else
+#include <mbedtls/ctr_drbg.h>
+#include <mbedtls/entropy.h>
+#endif
+#if MBEDTLS_VERSION_MAJOR < 3
 #include <mbedtls/certs.h>
+#endif
 #include <mbedtls/x509.h>
 #include <mbedtls/ssl.h>
 
@@ -55,6 +61,9 @@
 
 const int ciphers[] =
 {
+	MBEDTLS_TLS1_3_AES_256_GCM_SHA384,
+	MBEDTLS_TLS1_3_CHACHA20_POLY1305_SHA256,
+	MBEDTLS_TLS1_3_AES_128_GCM_SHA256,
 	MBEDTLS_TLS_ECDHE_RSA_WITH_AES_256_GCM_SHA384,
 	MBEDTLS_TLS_ECDHE_ECDSA_WITH_AES_256_GCM_SHA384,
 	MBEDTLS_TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256,
@@ -64,13 +73,32 @@ const int ciphers[] =
     0
 };
 
+#if MBEDTLS_VERSION_NUMBER >= 0x03000000
+#if !defined(MBEDTLS_USE_PSA_CRYPTO)
+#ifdef MBEDTLS_ENTROPY_C
+static mbedtls_entropy_context entropy;
+#ifdef MBEDTLS_CTR_DRBG_C
+static mbedtls_ctr_drbg_context ctr_drbg;
+#endif
+#endif
+#endif
+#endif
+
 static mbedtls_x509_crt certificate;
 static inline int x509parse_keyfile(mbedtls_pk_context *pk, const char *path, const char *pwd)
 {
     int ret;
 
     mbedtls_pk_init(pk);
+#if MBEDTLS_VERSION_NUMBER >= 0x03000000
+#if defined(MBEDTLS_USE_PSA_CRYPTO)
+    ret = mbedtls_pk_parse_keyfile(pk, path, pwd, mbedtls_psa_get_random, MBEDTLS_PSA_RANDOM_STATE);
+#else
+    ret = mbedtls_pk_parse_keyfile(pk, path, pwd, mbedtls_ctr_drbg_random, &ctr_drbg);
+#endif
+#else
     ret = mbedtls_pk_parse_keyfile(pk, path, pwd);
+#endif
     if (ret == 0 && !mbedtls_pk_can_do(pk, MBEDTLS_PK_ECDSA) && !mbedtls_pk_can_do(pk, MBEDTLS_PK_RSA))
 	{
         ret = MBEDTLS_ERR_PK_TYPE_MISMATCH;
@@ -127,6 +155,13 @@ static void initKey()
 #ifndef USE_MBEDTLS_HAVEGE
 int urandom_bytes(void *ctx, unsigned char *dest, size_t len)
 {
+#if (MBEDTLS_VERSION_MAJOR >= 3)
+#if defined(MBEDTLS_USE_PSA_CRYPTO)
+	mbedtls_psa_get_random(MBEDTLS_PSA_RANDOM_STATE, dest, len);
+#else
+	mbedtls_ctr_drbg_random(&ctr_drbg, dest, len);
+#endif
+#else
 	int cur;
 
 	while (len) {
@@ -135,6 +170,7 @@ int urandom_bytes(void *ctx, unsigned char *dest, size_t len)
 			continue;
 		len -= cur;
 	}
+#endif
 	return 0;
 }
 #endif
@@ -159,10 +195,20 @@ void SSLi_init(void)
 	/* Initialize random number generator */
 #ifdef USE_MBEDTLS_HAVEGE
 	mbedtls_havege_init(&hs);
+#else
+#if MBEDTLS_VERSION_NUMBER >= 0x03000000
+#if defined(MBEDTLS_USE_PSA_CRYPTO)
+	psa_crypto_init();
+#else
+	mbedtls_ctr_drbg_init(&ctr_drbg);
+	mbedtls_entropy_init(&entropy);
+	mbedtls_ctr_drbg_seed(&ctr_drbg, mbedtls_entropy_func, &entropy, NULL, 0);
+#endif
 #else
 	urandom_fd = open("/dev/urandom", O_RDONLY);
 	if (urandom_fd < 0)
 		Log_fatal("Cannot open /dev/urandom");
+#endif
 #endif
 
 	/* Initialize config */
@@ -187,7 +233,11 @@ void SSLi_init(void)
 #endif
 	mbedtls_ssl_conf_dbg(conf, pssl_debug, NULL);
 
+#if MBEDTLS_VERSION_NUMBER >= 0x03000000
+	mbedtls_ssl_conf_min_version(conf, MBEDTLS_SSL_MAJOR_VERSION_3, MBEDTLS_SSL_MINOR_VERSION_3);
+#else
 	mbedtls_ssl_conf_min_version(conf, MBEDTLS_SSL_MAJOR_VERSION_3, MBEDTLS_SSL_MINOR_VERSION_1);
+#endif
 
 	mbedtls_ssl_conf_ciphersuites(conf, (const int*)&ciphers);
 
@@ -196,7 +246,7 @@ void SSLi_init(void)
 	if((rc = mbedtls_ssl_conf_own_cert(conf, &certificate, &key)) != 0)
 		Log_fatal("mbedtls_ssl_conf_own_cert returned %d", rc);
 
-	Log_info("mbedTLS library version %s initialized", MBEDTLS_VERSION_STRING);
+	Log_info("Mbed TLS library initialized (version: %s)", MBEDTLS_VERSION_STRING);
 }
 
 void SSLi_deinit(void)
@@ -208,9 +258,16 @@ void SSLi_deinit(void)
 	
 #ifdef USE_MBEDTLS_HAVEGE
 	mbedtls_havege_free(&hs);
+#else
+#if MBEDTLS_VERSION_NUMBER >= 0x03000000
+#if !defined(MBEDTLS_USE_PSA_CRYPTO)
+	mbedtls_ctr_drbg_free(&ctr_drbg);
+	mbedtls_entropy_free(&entropy);
+#endif
 #else
 	close(urandom_fd);
 #endif
+#endif
 }
 
 bool_t SSLi_getSHA1Hash(SSL_handle_t *ssl, uint8_t *hash)
@@ -223,8 +280,15 @@ bool_t SSLi_getSHA1Hash(SSL_handle_t *ssl, uint8_t *hash)
 	}
 #if MBEDTLS_VERSION_NUMBER < 0x02070000L
 	mbedtls_sha1(cert->raw.p, cert->raw.len, hash);
-#else
+#elif MBEDTLS_VERSION_NUMBER < 0x03000000L
 	mbedtls_sha1_ret(cert->raw.p, cert->raw.len, hash);
+#elif !defined(MBEDTLS_USE_PSA_CRYPTO)
+	mbedtls_sha1(cert->raw.p, cert->raw.len, hash);
+#else
+	size_t hash_length;
+	mbedtls_psa_hash_compute(
+		PSA_ALG_SHA_1, cert->raw.p, cert->raw.len, hash,
+		20 /* client_t member uint8_t hash[20] */, &hash_length);
 #endif
 	return true;
 }
